/*
PRICE was written by Graham Ruby.
This file is part of PRICE.

PRICE is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

PRICE is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with PRICE.  If not, see <http://www.gnu.org/licenses/>.
*/


#ifndef SCOREDSEQSUBSEQ_CPP
#define SCOREDSEQSUBSEQ_CPP

#include "ScoredSeqSubseq.h"
#include "ScoredSeqShallow.h"

long ScoredSeqSubseq::_readCount = 0;

ScoredSeqSubseq::ScoredSeqSubseq(){} //default constructor

ScoredSeqSubseq::ScoredSeqSubseq(ScoredSeq* innerSeq, long offset, long length) :
  _offset(offset),
  _size(length),
  _innerSeq(innerSeq){
  if (_offset < 0 or _size < 1 or _offset + _size > _innerSeq->size()){
    throw AssemblyException::ArgError("ScoredSeqSubseq is beyond the scope of its inner sequence");
  }
  _minusOffset = _innerSeq->size() - _size - _offset;
  //_bufferSeq = 0;
  //_readCount++;
}

ScoredSeqSubseq::~ScoredSeqSubseq(){
  //_readCount--;
  unbuffer();
}



float ScoredSeqSubseq::scoreAtPosition(long position, char sense) {
  //if (_bufferSeq != 0){ return _bufferSeq->scoreAtPosition(position,sense); }
  //else {
    if (sense=='+'){ position += _offset; }
    else if (sense=='-'){ position += _minusOffset; }
    else { throw AssemblyException::ArgError("sense must be (+) or (-)"); }
    return _innerSeq->scoreAtPosition(position,sense);
  //}
}

float ScoredSeqSubseq::linkAfterPosition(long position, char sense) {
  //if (_bufferSeq != 0){ return _bufferSeq->linkAfterPosition(position,sense); }
  //else {
    if (sense=='+'){ position += _offset; }
    else if (sense=='-'){ position += _minusOffset; }
    else { throw AssemblyException::ArgError("sense must be (+) or (-)"); }
    return _innerSeq->linkAfterPosition(position,sense);
  //}
}


char ScoredSeqSubseq::nucAtPosition(long position, char sense) {
  //if (_bufferSeq != 0){ return _bufferSeq->nucAtPosition(position,sense); }
  //else {
    if (sense=='+'){ position += _offset; }
    else if (sense=='-'){ position += _minusOffset; }
    else { throw AssemblyException::ArgError("sense must be (+) or (-)"); }
    return _innerSeq->nucAtPosition(position,sense);
  //}
}

string ScoredSeqSubseq::getSeq(char sense) {
  //if (_bufferSeq != 0){ return _bufferSeq->getSeq(sense); }
  //else {
    long offset;
    if (sense=='+'){ offset = _offset; }
    else if (sense=='-'){ offset = _minusOffset; }
    else { throw AssemblyException::ArgError("sense must be (+) or (-)"); }
    return _innerSeq->getSubseq(offset, _size, sense);
  //}
}
string ScoredSeqSubseq::getSubseq(long position, long length, char sense){
  //if (_bufferSeq != 0){ return _bufferSeq->getSubseq(position,length,sense); }
  //else {
    if (sense=='+'){ position += _offset; }
    else if (sense=='-'){ position += _minusOffset; }
    else { throw AssemblyException::ArgError("sense must be (+) or (-)"); }
    return _innerSeq->getSubseq(position,length,sense);
  //}
}



long ScoredSeqSubseq::size() {
  return _size;
}

void ScoredSeqSubseq::buffer(){
  //if (_bufferSeq == 0){ _bufferSeq = new ScoredSeqShallow(this,'+'); }
}
void ScoredSeqSubseq::unbuffer(){
  //delete _bufferSeq;
  //_bufferSeq = 0;
}
void ScoredSeqSubseq::bottomBuffer(){
  _innerSeq->bottomBuffer();
}


bool ScoredSeqSubseq::hasPairedEnd() {
  return false;
}

ScoredSeq * ScoredSeqSubseq::getPairedEnd() {
  throw AssemblyException::CallingError("subseq has no paired end");
}
ScoredSeq * ScoredSeqSubseq::getTempPairedEnd() {
  throw AssemblyException::CallingError("subseq has no paired end");
}

ScoredSeq * ScoredSeqSubseq::shallowCopy(){
  return new ScoredSeqShallow(this,'+');
}
ScoredSeq * ScoredSeqSubseq::flipCopy(){
  return new ScoredSeqShallow(this,'-');
}


bool ScoredSeqSubseq::isNested(){ return true; }
ScoredSeq* ScoredSeqSubseq::getNested(){ return _innerSeq; }
void ScoredSeqSubseq::deepDelete(){
  _innerSeq->deepDelete();
  delete this;
}
void ScoredSeqSubseq::deepUnbuffer(){
  unbuffer();
  _innerSeq->deepUnbuffer();
}


#endif
